% Estimation example for sparse regression example using glm-ie.
%
% We use the Nordborg Flowering Time dataset from http://walnut.usc.edu/.
%   X is a binary matrix of 166 individuals x 5k SNPs selected by prior
%     knowledge
%   y is a real vector with flowering times between 21 and 200
%
% (c) by Hannes Nickisch, MPI for Biological Cybernetics, 2010 July 22
clear all, close all

load regress, y = (y-mean(y))/std(y); [m,n] = size(X);                % get data

% Gaussian or non-Gaussian i.e. sparse weight prior
% select by uncommenting the desired one
% pot = @(s) potT(s,2);            % Student's t
% pot = @(s) potLaplace(s);        % Laplace
pot = @(s) potExpPower(s,1.3);   % Exponential Power family

% Numbers from Christoph
s2 = 0.0078; B = 1;  tau = 110;         % noise variance and sparse weight prior

fprintf('Do sparse regression using a (sparse) '), str = func2str(pot);
fprintf('%s weight prior.\n',str(8:end))

% optimisation parameters
opt.nMVM = 250; opt.output = 1; opt.exactNewt = 1;
u0 = zeros(n,1);

% apply 4 different PLS schemes
plsList = {'TN','CGBT','CG','LBFGS'};
pars = {u0,X,y,B,opt,s2,'penFromPot',pot,tau};
for i=1:length(plsList)
  fprintf('PLS optimisation using %s.\n',plsList{i})
  tic, [u{i},phi(i)] = feval(['pls',plsList{i}],pars{:}); t(i) = toc;
  res(i) = norm(X*u{i}-y)/norm(y);
end

fprintf('residual in %%\n')
for i=1:length(plsList), fprintf('%s %1.3e%%, ',plsList{i},res(i)), end
fprintf('\b\b\n')

fprintf('running times\n')
for i=1:length(plsList), fprintf('%s %1.2fs, ',plsList{i},t(i)), end
fprintf('\b\b\n')

fprintf('objective function values\n')
for i=1:length(plsList), fprintf('%s %1.4e, ',plsList{i},phi(i)), end
fprintf('\b\b\n')