% wavelet transform matrix: constructor
%
% A = wavmat(sz,qmf,nl)
%
%  sz  [1,2]: size of the tensor to transform
%  qmf [1,p]: quadrature mirror filter, default is Haar qmf = [1,1]/sqrt(2)
%  nl  [1,1]: number of layers,         default is full nl
%
% (c) by Hannes Nickisch, MPI for Biological Cybernetics, 2010 July 15

function A = wavmat(sz,qmf,nl)

  imsz = sz(:)';  A.imsz = imsz;                                    % image size
  if numel(A.imsz)==1, A.imsz = [A.imsz,1]; end
  sz = prod(imsz)*[1,1];                                           % matrix size
  
  % wavelet type
  if nargin>1
    A.qmf = qmf;
  else
    A.qmf = [1,1]/sqrt(2);                                     % default is Haar
  end
  if nargin>2
    A.nl  = nl;
  else
    imsz = imsz/2;
    for nl = 1:min(fix(log2(imsz)))
        if any(imsz ~= fix(imsz)), nl = nl-1; break, end
        imsz = imsz/2;
    end
    A.nl = nl;                                                 % default is full
  end
    
  A.sz = sz; A.ctransp = 0; A = class(A,mfilename);                  % [generic]